<?php

namespace Goldenscarab\Modulus\Debusca\App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Config;
use Goldenscarab\Modulus\Debusca\App\Http\Services\Debusca\Facade\DebuscaService;

class BddController extends Controller
{
    private $db;
    private $connexion;

    public function __construct(Request $request)
    {
        $db_name = $request->get('db_name');
        $bdd = DebuscaService::getDatabases();

        $db_config = $bdd->where('DB_NAME', $db_name)->first();

        if (is_null($db_config)) {
            $db_config = $bdd->first();
        }

        // Récupération de la config par défaut du driver
        $default_config_driver = Config::get('database.connections.' . $db_config['DB_CONNECTION']);

        // Construction de la nouvelle configuration
        $new_config = array_merge($default_config_driver, [
            'driver'    => $db_config['DB_CONNECTION'],
            'host'      => $db_config['DB_HOST'],
            "port"      => $db_config['DB_PORT'],
            'database'  => $db_config['DB_DATABASE'],
            'username'  => $db_config['DB_USERNAME'],
            'password'  => $db_config['DB_PASSWORD'],
            'options' => [
                'MultipleActiveResultSets' => false
            ]
        ]);

        //dd($default_config_driver, $new_config);
        Config::set('database.connections.' . $db_config['DB_NAME'], $new_config);

        // Mise à jour de la config BDD
        $this->db = DB::connection($db_config['DB_NAME']);
        $driver_name = ucfirst(strtolower($db_config['DB_CONNECTION']));
        $driver_class = '\Goldenscarab\Modulus\Debusca\App\Http\Drivers\\' . $driver_name;

        $this->connexion = new $driver_class($this->db);
    }

    public function index()
    {
        $databases = DebuscaService::getDatabases();

        return view('debusca::app.bdd.index')->with([
            'title'     => "Debusca",
            'subtitle'  => "BDD",
            'databases' => $databases
        ]);
    }

    public function tables(Request $request)
    {
        $cache = $request->get('cache') == 'true';
        $search = $request->get('search');

        if (!$cache) {
            Cache::flush();
        }

        $result = $this->connexion->getTables();

        // Filtrer les tables
        $result = $result->filter(function($item) use ($search) {
            $regex = sprintf('/.*%s.*/i', $search);
            return preg_match($regex, $item->getResult()->first()->getValues()) > 0;
        });

        return view('debusca::app.bdd.partials._tree')->with([
            'result' => $result,
        ]);
    }

    public function suggestions(Request $request)
    {
         $suggestions = $this->connexion->getSuggestions();

        return response()->json($suggestions);
    }

    public function sendQuery(Request $request)
    {
        $query = $request->get('query');
        $result = $this->connexion->query($query);

        // Limite du résultat aux 200 premiers éléments
        $limit_result = $result->getResult()->take(200);
        $result->setResult($limit_result);

        return view('debusca::app.bdd.partials._result')->with([
            'result' => $result,
        ]);
    }

    public function getQueryString(Request $request)
    {
        $query = $request->get('query');

        $string = $this->connexion->getQueryString($query);

        return response()->json(['query' => $string]);
    }

}
