<?php

namespace Goldenscarab\Modulus\Debusca\App\Http\Services\Debusca;

use Exception;
use Throwable;
use RuntimeException;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Blade;
use Symfony\Component\Finder\SplFileInfo;

class DebuscaService
{

    public function find($path, $search)
    {
        // Ajout des fichiers du dossier courant
        $all_files = File::allFiles($path, true);
        $filtered_files = collect($all_files)->filter(function($file) use ($search) {

            $str_search   = strtolower($search);
            $str_filename = strtolower($file->getBaseName());

            return preg_match('/'.preg_quote($str_search).'/i', $str_filename) > 0;
        });

        return $filtered_files;
    }



    public function grep($path, $search, $allowed_exts = ['*'])
    {
        $files = File::allFiles($path);
        $finded = collect();

        foreach ($files as $file) {
            if ($this->isFileTypeAllowed($file, $allowed_exts)) {

                if ($file->isReadable()) {
                    $occurences = $this->findInFile($file, $search);

                    if (count($occurences) > 0) {
                        $finded->push([
                            'file' => $file,
                            'occurences' => $occurences
                        ]);
                    }

                }
            }
        }

        return $finded;

    }

    private function isFileTypeAllowed(SplFileInfo $file, $allowed_exts = ['*'])
	{
        if (in_array('*', $allowed_exts)) {
			return true;
		}

        $extension = $file->getExtension();

        return in_array($extension, $allowed_exts) ? true : false;
	}


    private function findInFile(SplFileInfo $file, $search)
	{
        $finded       = [];
        $line         = 1;

        try {

            $fp = fopen($file->getPathname(), 'r');

            while (($buffer = fgets($fp)) !== false) {

                if (preg_match('|' . preg_quote(strtolower($search)) . '|', strtolower($buffer))) {

                    if (strlen($buffer) > 200) {
                        $buffer = substr($buffer, 0, 200) . '...';
                    }

                    $finded[$line] = trim(htmlspecialchars_decode(htmlspecialchars($buffer)));
                }
                $line++;
            }

		} catch (Exception $e) {
			throw new Exception('Impossible de lire le fichier : ' . $file);
		}

        return $finded;

	}

    public function getDatabases()
    {
        $db = DB::connection();
        $default = [
            "DB_NAME"       => 'APPLI',
            "DB_CONNECTION" => $db->getConfig('driver'),
            "DB_HOST"       => $db->getConfig('host'),
            "DB_PORT"       => $db->getConfig('port'),
            "DB_DATABASE"   => $db->getConfig('database'),
            "DB_USERNAME"   => $db->getConfig('username'),
            "DB_PASSWORD"   => $db->getConfig('password')
        ];

        $databases = collect();
        $databases->push($default);

        if (File::exists(debusca_path('config/bdd.json'))) {
            $others = json_decode(File::get(debusca_path('config/bdd.json')), true);
        } else {
            $others = [];
        }

        $databases = $databases->merge(collect($others));

        return $databases;
    }
}
