@include('debusca::app.file.partials._terminal-theme')
<script>

    class DesbucaFile {

        constructor(current_path) {
            this.current_path = current_path;

            this.init();
        }

        init() {
            this.initEditor();
            this.initListeners();

            this.updateTreeHtml(this.formatPath(this.current_path));
            this.updateBreadCrumbHtml(this.formatPath(this.current_path));
            this.updateEditPathName(this.formatPath(this.current_path) + '/');
        }

        initEditor() {

            var _this = this;

            // Editeur
            var editor = ace.edit("editor");
            editor.setTheme("ace/theme/monokai");
            editor.session.setMode("ace/mode/javascript");
            editor.setOptions({
                enableEmmet: true,
                fontFamily: 'monospace',
                fontSize: '12px'
            });
            editor.getSession().setUseWrapMode(true);
            editor.setHighlightActiveLine(true);
            editor.setShowPrintMargin(false);

            // TODO - Revoir une procédure pour afficher le dernier fichier edité
            // var old_value = localStorage.getItem('debusca.file.editor')
            // if (old_value) {
            //     editor.setValue(old_value);
            //     editor.selection.clearSelection();
            // }

            editor.getSession().on('change', function(e) {
                //localStorage.setItem('debusca.file.editor', editor.getValue());
            });

            var terminal = ace.edit("terminal");
            terminal.setTheme("ace/theme/cterminal");
            terminal.session.setMode("ace/mode/csh");
            terminal.getSession().setUseWrapMode(true);
            terminal.setFontSize("16px");
            terminal.setOptions({
                printMargin: false,
                highlightActiveLine: false,
                showLineNumbers: false,
                showGutter: false,
                scrollPastEnd: true,
                showFoldWidgets: false,
                cursorStyle: "wide",
            });

            // Uniquement la dernière ligne editable
            terminal.commands.on("exec", function(e) {
                var rowCol = terminal.selection.getCursor();
                if (rowCol.row < (terminal.session.getLength() - 1)) {
                    e.preventDefault();
                    e.stopPropagation();
                }

                var currline = terminal.getSelectionRange().start.row;
                var linetext = terminal.session.getLine(currline);
                var matches  = linetext.match(/[^\$]*\$\s(.*)/i);
                var command  = '';
                var start    = {row: currline, column: 0};
                var end      = terminal.getCursorPosition();

                if (matches && matches.length > 1) {
                    command = matches[1];
                }

                // Validation d'un commande
                if (e.args == "\n") {
                    _this.terminalCommand(command, 'enter');
                }

                if (e.command.name == 'golineup') {
                    // Historique précédent
                    e.preventDefault();
                    e.stopPropagation();

                    terminal.session.remove({start: start, end: end});
                    _this.terminalCommand(command, 'up');

                } else if (e.command.name == 'golinedown') {
                    // Historique suivant
                    e.preventDefault();
                    e.stopPropagation();

                    terminal.session.remove({start: start, end: end});
                    _this.terminalCommand(command, 'down');
                } else if (e.command.name == 'indent') {
                    // Suggestion
                    e.preventDefault();
                    e.stopPropagation();
                    console.log(command);
                    _this.terminalCommand(command, 'tab');
                } else if (e.command.name == 'backspace') {
                    // Ne pas supprimer le prompt
                    if (!command) {
                        e.preventDefault();
                        e.stopPropagation();
                    }
                }

            });

            this.editor = editor;
            this.terminal = terminal;
        }

        initListeners() {

            let _this = this;

            // Mise à jour de l'explorateur
            $(document).on('click', '.item-folder, .breadcrumb-folder', function() {
                var path = $(this).attr('data-path');
                _this.current_path = path;
                _this.updateTreeHtml(_this.formatPath(path));
                _this.updateBreadCrumbHtml(_this.formatPath(path));
                _this.updateEditPathName(_this.formatPath(path) + '/')
            });

            // Edition d'un fichier
            $(document).on('click', '.item-file', function() {
                $('.item-file').removeClass('active');
                $(this).addClass('active');
                var file = $(this).attr('data-path');
                _this.updateEditPathName(file);
                _this.updateEditor(file);
            });

            // A l'enregistrement du fichier
            $("#form-editor").on('submit', function(e) {
                e.preventDefault();

                var value = _this.editor.getValue().trim();
                var file = $('#field-current-filename').val();
                var path = file.match(/^\/.*\//i)[0];

                if (file != path) {
                    var command = '';
                    command += "cat << 'EOT' > '" + file + "' \n";
                    command += value + "\n";
                    command += "EOT \n";

                    $('#btn-save-file').find('.fa').removeClass('fa-floppy-o').addClass('fa-spinner rotate');

                    $.ajax({
                        url      : "{{ route('debusca.command.exec') }}",
                        headers  : { 'X-CSRF-Token' : window.Laravel.csrfToken },
                        type     : 'POST',
                        data     : {'pwd': _this.current_path, 'command': command },
                        success  : function(result) {
                            $('#btn-save-file').find('.fa').removeClass('fa-spinner rotate').addClass('fa-floppy-o');
                            _this.updateTreeHtml(_this.formatPath(_this.current_path));
                            _this.updateEditor(file);
                            $('#btn-rename-file').removeClass('disabled');
                            $('#btn-delete-file').removeClass('disabled');

                            // Message alert
                            Toast.fire({
                                icon: 'success',
                                title: "Fichier enregistré avec succès"
                            })
                        },
                        error    : function(result){
                            console.log(result)
                            Toast.fire({
                                icon: 'error',
                                title: "Impossible d'enregistrer le fichier",
                                text: result.responseJSON.message
                            })
                        },
                    });

                } else {
                    Toast.fire({
                        icon: 'error',
                        title: "Un nom de fichier doit être spécifié avant d'enregistrer"
                    })
                }
            });

            // Renommage d'un fichier
            $('#btn-rename-file').on('click', function() {
                if (!$(this).hasClass('disabled') && confirm('Veuillez confirmer le renommage de ce fichier')) {
                    var old_name = $('#field-original-filename').val();
                    var new_name = $('#field-current-filename').val();
                    var new_path = new_name.match(/^\/.*\//i)[0].replace(/\/$/, '');

                    var command = 'mv "' + old_name + '" "' + new_name + '"';

                    $.ajax({
                        url      : "{{ route('debusca.command.exec') }}",
                        headers  : { 'X-CSRF-Token' : window.Laravel.csrfToken },
                        type     : 'POST',
                        data     : {'pwd': _this.current_path, 'command': command },
                        success  : function(result){
                            _this.current_path = new_path;
                            _this.updateTreeHtml(_this.formatPath(_this.current_path));
                            _this.updateBreadCrumbHtml(_this.formatPath(_this.current_path));
                            _this.updateEditPathName(new_name);

                            // Message alert
                            Toast.fire({
                                icon: 'success',
                                title: "Élément renommé avec succès"
                            })

                        },
                        error    : function(result){
                            console.log(result)
                            Toast.fire({
                                icon: 'error',
                                title: "Impossible de dupliquer le fichier",
                                text: result.responseJSON.message
                            })
                        },
                    });
                }
            });

            // Actualiser l'explorateur
            $(document).on('click', '.tree-refresh', function(e) {
                _this.updateTreeHtml(_this.formatPath(_this.current_path));
            });

            // Nouveau fichier
            $(document).on('click', '.create-file', function(e) {
                var command = 'touch "nouveau-fichier.txt"';

                $.ajax({
                    url      : "{{ route('debusca.command.exec') }}",
                    headers  : { 'X-CSRF-Token' : window.Laravel.csrfToken },
                    type     : 'POST',
                    data     : {'pwd': _this.current_path, 'command': command },
                    success  : function(result){
                        _this.updateTreeHtml(_this.formatPath(_this.current_path));

                        // Message alert
                        Toast.fire({
                            icon: 'success',
                            title: "Fichier créé avec succès"
                        })
                    },
                    error    : function(result){
                        console.log(result)
                        Toast.fire({
                            icon: 'error',
                            title: "Impossible de créer le fichier",
                            text: result.responseJSON.message
                        })
                    },
                });
            });

            // Nouveau dossier
            $(document).on('click', '.create-folder', function(e) {
                var command = 'mkdir "Nouveau dossier"';

                $.ajax({
                    url      : "{{ route('debusca.command.exec') }}",
                    headers  : { 'X-CSRF-Token' : window.Laravel.csrfToken },
                    type     : 'POST',
                    data     : {'pwd': _this.current_path, 'command': command },
                    success  : function(result){
                        _this.updateTreeHtml(_this.formatPath(_this.current_path));

                        // Message alert
                        Toast.fire({
                            icon: 'success',
                            title: "Dossier créé avec succès"
                        })
                    },
                    error    : function(result){
                        console.log(result)
                        Toast.fire({
                            icon: 'error',
                            title: "Impossible de créer le dossier",
                            text: result.responseJSON.message
                        })
                    },
                });
            });

            // Suppression d'un fichier
            $(document).on('click', '.delete-file', function(e) {
                e.stopPropagation();
                if (!$(this).hasClass('disabled') && confirm('Veuillez confirmer la suppression de ce fichier')) {
                    var file = $(this).closest('.item-file').attr('data-path');
                    //var path = file.match(/^\/.*\//i)[0];
                    var command = 'rm "' + file + '"';

                    $.ajax({
                        url      : "{{ route('debusca.command.exec') }}",
                        headers  : { 'X-CSRF-Token' : window.Laravel.csrfToken },
                        type     : 'POST',
                        data     : {'pwd': _this.current_path, 'command': command },
                        success  : function(result){
                            _this.updateTreeHtml(_this.formatPath(_this.current_path));

                            // Message alert
                            Toast.fire({
                                icon: 'success',
                                title: "Fichier supprimé avec succès"
                            })
                        },
                        error    : function(result){
                            Toast.fire({
                                icon: 'error',
                                title: "Impossible de supprimer le fichier",
                                text: result.responseJSON.message
                            })
                        },
                    });
                }
            });

            // Suppression d'un dossier
            $(document).on('click', '.delete-folder', function(e) {
                e.stopPropagation();
                if (!$(this).hasClass('disabled') && confirm('Veuillez confirmer la suppression de ce dossier')) {
                    var folder = $(this).closest('.item-folder').attr('data-path');
                    var command = 'rm -rf "' + folder + '"';

                    $.ajax({
                        url      : "{{ route('debusca.command.exec') }}",
                        headers  : { 'X-CSRF-Token' : window.Laravel.csrfToken },
                        type     : 'POST',
                        data     : {'pwd': _this.current_path, 'command': command },
                        success  : function(result){
                            _this.updateTreeHtml(_this.formatPath(_this.current_path));

                            // Message alert
                            Toast.fire({
                                icon: 'success',
                                title: "Dossier supprimé avec succès"
                            })
                        },
                        error    : function(result){
                            Toast.fire({
                                icon: 'error',
                                title: "Impossible de supprimer le dossier",
                                text: result.responseJSON.message
                            })
                        },
                    });
                }
            });

            // Téléchargement d'un fichier
            $(document).on('click', '.download', function(e) {
                e.stopPropagation();
                if (!$(this).hasClass('disabled') && confirm('Veuillez confirmer le téléchargement de ce fichier')) {
                    var pathname = $(this).closest('.item-file').attr('data-path');
                    var file = pathname.match(/.*\/([^\/]*)$/i)[1];

                    $.ajax({
                        url      : "{{ route('debusca.file.download') }}",
                        type     : 'GET',
                        data     : {'pathname': pathname },
                        xhrFields: { responseType: 'blob' },
                        success  : function(result){
                            console.log(result);
                            var a = document.createElement('a');
                            var url = window.URL.createObjectURL(result);
                            a.href = url;
                            a.download = file;
                            document.body.append(a);
                            a.click();
                            a.remove();
                            window.URL.revokeObjectURL(url);
                        },
                        error    : function(result){
                            Toast.fire({
                                icon: 'error',
                                title: "Impossible de télécharger le fichier",
                                text: result.responseJSON.message
                            })
                        },
                    });
                }
            });

            // Création d'une archive de dossier
            $(document).on('click', '.archive-folder', function(e) {
                e.stopPropagation();
                if (!$(this).hasClass('disabled') && confirm('Veuillez confirmer la création d\'une archive de ce dossier')) {
                    var path_folder = $(this).closest('.item-folder').attr('data-path');

                    var curr_path = path_folder.match(/(.*)\/[^\/]*$/i)[1];
                    var folder    = path_folder.match(/.*\/([^\/]*)$/i)[1];

                    var command = 'tar -cf "' + path_folder + '.tar" -C "' + curr_path + '" "' + folder + '"';

                    $.ajax({
                        url      : "{{ route('debusca.command.exec') }}",
                        headers  : { 'X-CSRF-Token' : window.Laravel.csrfToken },
                        type     : 'POST',
                        data     : {'pwd': _this.current_path, 'command': command },
                        success  : function(result){
                            _this.updateTreeHtml(_this.formatPath(_this.current_path));

                            // Message alert
                            Toast.fire({
                                icon: 'success',
                                title: "Archive créée avec succès"
                            })
                        },
                        error    : function(result){
                            Toast.fire({
                                icon: 'error',
                                title: "Impossible de créer l'archive",
                                text: result.responseJSON.message
                            })
                        },
                    });
                }
            });

            // Upload de fichier
            $(document).on('click', '.file-upload', function() {
                $('<input>', {type: 'file'}).trigger("click").on('change', function(e) {

                    var form = new FormData();
                    form.append('path', _this.current_path);
                    form.append('file', e.target.files[0]);

                    $.ajax({
                        xhr: function() {
                            var setXhr = $.ajaxSettings.xhr();
                            if(setXhr.upload) {
                                setXhr.upload.addEventListener('progress', function(event) {
                                    var sizeTotal  = event.total;
                                    var sizeLoaded = event.loaded;
                                    var percentage = 100 - Math.ceil((sizeLoaded * 100) / sizeTotal);
                                    console.log(percentage + "%");
                                }, false);
                            }
                            return setXhr;
                        },
                        url        : "{{ route('debusca.file.upload') }}",
                        headers  : { 'X-CSRF-Token' : window.Laravel.csrfToken },
                        type       : 'POST',
                        contentType: false,
                        processData: false,
                        dataType   : 'json',
                        data       : form,
                        success    : function(result){
                            _this.updateTreeHtml(_this.formatPath(_this.current_path));

                            // Message alert
                            Toast.fire({
                                icon: 'success',
                                title: result.success
                            })
                        },
                        error      : function(result){
                            console.log(result)
                            Toast.fire({
                                icon: 'error',
                                title: "Impossible d'ajouter le fichier'",
                                text: result.responseJSON ? result.responseJSON.message : 'Pas de message'
                            })
                        },
                    });
                });
            });

            // Recherche de fichiers
            $('#btn-search-files').on('click', function() {
                var search = $('#field-search-files').val();
                $('#field-search-contents').val('');
                _this.searchFiles(search)
            });
            $('#field-search-files').on('keyup', function(e) {
                if(e.keyCode == 13) {
                    var search = $('#field-search-files').val();
                    $('#field-search-contents').val('');
                    _this.searchFiles(search)
                }
            });

            // Recherche de contenu
            $('#btn-search-contents').on('click', function() {
                var search = $('#field-search-contents').val();
                $('#field-search-files').val('');
                _this.searchContents(search)
            });
            $('#field-search-contents').on('keyup', function(e) {
                if(e.keyCode == 13) {
                    var search = $('#field-search-contents').val();
                    $('#field-search-files').val('');
                    _this.searchContents(search)
                }
            });

            // Ouverture du terminal
            $('#btn-terminal').on('click', function() {
                _this.terminal.session.setValue('');
                _this.terminalCommand(null, null, _this.current_path);
            });
        }

        searchFiles(search) {
            if (search) {
                this.updateTreeFileFindHtml(this.formatPath(this.current_path), search);
                this.updateBreadCrumbHtml(this.formatPath(this.current_path) + '/*');
            } else {
                this.updateTreeHtml(this.formatPath(this.current_path));
                this.updateBreadCrumbHtml(this.formatPath(this.current_path));
            }
        }

        searchContents(search) {
            if (search) {
                this.updateTreeFileGrepHtml(this.formatPath(this.current_path), search);
                this.updateBreadCrumbHtml(this.formatPath(this.current_path) + '/*');
            } else {
                this.updateTreeHtml(this.formatPath(this.current_path));
                this.updateBreadCrumbHtml(this.formatPath(this.current_path));
            }
        }

        updateEditor(file) {

            var _this = this;

            var ext = null;
            if (file.match(/\.([0-9a-z]+)$/i)) {
                ext  = file.match(/\.([0-9a-z]+)$/i)[1];
            }

            var editor_mode = {
                'php': 'php',
                'js': 'javascript',
                'htm': 'html',
                'html': 'html',
                'xml': 'xml',
                'sql': 'sql',
                'java': 'java',
                'md': 'markdown',
                'c': 'c_cpp',
                'cpp': 'c_cpp',
                'sh': 'batchfile',
                'css': 'css',
                'json': 'json',
                'less': 'less',
                'scss': 'scss',
                'yml': 'yaml',
            }

            var mode = editor_mode[ext] ? editor_mode[ext] : 'text';

            _this.editor.session.setMode("ace/mode/" + mode);
            var pos = _this.editor.getCursorPosition();

            $.ajax({
                url      : "{{ route('debusca.file.content') }}",
                type     : 'GET',
                data     : {'pathname' : file },
                success  : function(result){
                    _this.editor.setValue(result, -1);
                    $('#btn-rename-file').removeClass('disabled');
                    $('#btn-delete-file').removeClass('disabled');

                    _this.editor.moveCursorTo(pos.row, pos.column);
                },
                error    : function(result){
                    console.log(result)
                    Toast.fire({
                        icon: 'error',
                        title: "Impossible de charger le fichier",
                        text: result.responseJSON.message
                    })
                },
            });
        }

        updateTreeHtml(path) {
            $.ajax({
                url      : "{{ route('debusca.file.tree') }}",
                type     : 'GET',
                data     : {'path' : path },
                success  : function(result){
                    $('#explorer').html(result);
                },
                error    : function(result){
                    console.log(result)
                    Toast.fire({
                        icon: 'error',
                        title: 'Impossible de charger le contenu du dossier : ' + path,
                        text: result.responseJSON.message
                    })
                },
            });
        }

        updateTreeFileFindHtml(path, search) {
            $('#btn-search-files .fa').removeClass('fa-search').addClass('fa-spinner rotate');
            var render = '';
            $.ajax({
                url      : "{{ route('debusca.file.find') }}",
                headers  : { 'X-CSRF-Token' : window.Laravel.csrfToken },
                type     : 'POST',
                data     : {'search' : search, 'path': path},
                success  : function(result){
                    $('#explorer').html(result);
                    $('#btn-search-files .fa').removeClass('fa-spinner rotate').addClass('fa-search');
                },
                error    : function(result){
                    console.log(result)
                    Toast.fire({
                        icon: 'error',
                        title: 'Impossible de rechercher le fichier : ' + search,
                        text: result.responseJSON.message
                    })
                },
            });
        }

        terminalCommand(command, key, pwd) {
            var _this = this;

            $.ajax({
                url      : "{{ route('debusca.command.terminal') }}",
                headers  : { 'X-CSRF-Token' : window.Laravel.csrfToken },
                type     : 'POST',
                data     : {'command': command, 'key': key, 'pwd': pwd },
                async    : true,
                success  : function(result){
                    _this.terminal.session.insert(_this.terminal.getCursorPosition(), result);
                    var currline = _this.terminal.getSelectionRange().start.row - 10;
                    _this.terminal.scrollToLine(currline, true, true, function () {});
                },
                error    : function(result){
                    console.log(result)
                    Toast.fire({
                        icon: 'error',
                        title: 'Une erreur est survenu lors de l\'exécution de la commande',
                        text: result.responseJSON.message
                    })
                },
            });
        }

        updateTreeFileGrepHtml(path, search) {
            var _this = this;
            $('#btn-search-contents .fa').removeClass('fa-search').addClass('fa-spinner rotate');
            var render = '';
            var pos = _this.editor.getCursorPosition();
            $.ajax({
                url      : "{{ route('debusca.file.grep') }}",
                headers  : { 'X-CSRF-Token' : window.Laravel.csrfToken },
                type     : 'POST',
                data     : {'search' : search, 'path': path},
                success  : function(result){
                    $('#explorer').html(result.explorer);
                    _this.editor.session.setMode("ace/mode/text");
                    _this.editor.setValue(result.editor, -1);
                    _this.editor.moveCursorTo(pos.row, pos.column);
                    $('#btn-search-contents .fa').removeClass('fa-spinner rotate').addClass('fa-search');
                },
                error    : function(result){
                    console.log(result)
                    Toast.fire({
                        icon: 'error',
                        title: 'Impossible de rechercher le fichier : ' + search,
                        text: result.responseJSON.message
                    })
                },
            });
        }

        updateBreadCrumbHtml(path) {
            if (path) {

                if (!/\/$/.test(path)) {
                    path = path + '/';
                }

                var splited_path = path.split('/');

                var $li = $('<li>').addClass('breadcrumb-item');
                var $breadcrumb = $('<ol>').addClass('breadcrumb');
                var curr_path = '';

                $.each(splited_path, function (key, item) {
                    if (key < (splited_path.length - 2)) {

                        curr_path += curr_path == '/' ? item : ('/' + item);
                        var $a = $('<a>', { href: '#' }).attr('data-path', curr_path).addClass('breadcrumb-folder').text(item);
                        var $item = $li.clone().append($a);
                        $breadcrumb.append($item);

                    } else {
                        var $item = $li.clone().text(item)
                        $breadcrumb.append($item);
                    }
                });

                $('#breadcrumb').html($breadcrumb);
            }
        }

        updateEditPathName(pathname) {
            $('#field-original-filename').val(pathname)
            $('#field-current-filename').val(pathname)
        }

        formatPath(path) {

            var curr_path = '';

            if (path) {
                var splited_path = path.split('/');
                $.each(splited_path, function (key, folder) {
                    if (folder != '..' && splited_path[key +1 ] != '..') {
                        if (curr_path[curr_path.length -1] != '/') {
                            curr_path += '/';
                        }
                        curr_path += folder;
                    }
                });
            }

            return curr_path;
        }


    }
</script>
